#!/bin/bash
# This is a script modified from Picasa wrapper script to launch WinRARSetup.exe

cd `dirname "$0"`
winrar_root=`pwd`

# Wine environment required
export PATH="$winrar_root/Platform/bin":"$winrar_root":$PATH
export LD_LIBRARY_PATH="$winrar_root/Platform/lib":"$winrar_root/lib":$LD_LIBRARY_PATH
export WINEDLLPATH="$winrar_root/Platform/lib/wine"
export WINELOADER="$winrar_root/Platform/bin/wine"
export WINESERVER="$winrar_root/Platform/bin/wineserver"
export WINEPREFIX=$HOME/.winrar
export WINEDLLOVERRIDES="msxml3=n" # use native msxml3
# find the x window server location at [current machine]:[displaynumber=0]:[screennumber=0] to run x window application
# used in crontab schedule part of winrar
export DISPLAY=":0.0"

# create directories
[ -d "$WINEPREFIX" ] || mkdir -p "$WINEPREFIX"
[ -d "$WINEPREFIX/drive_c" ] || mkdir -p "$WINEPREFIX/drive_c"
[ -d "$WINEPREFIX/drive_c/Program Files" ] || mkdir -p "$WINEPREFIX/drive_c/Program Files"
[ -d "$WINEPREFIX/drive_c/windows" ] || mkdir -p "$WINEPREFIX/drive_c/windows"
[ -d "$WINEPREFIX/drive_c/windows/temp" ] || mkdir -p "$WINEPREFIX/drive_c/windows/temp"
[ -d "$WINEPREFIX/drive_c/windows/inf.done" ] || mkdir -p "$WINEPREFIX/drive_c/windows/inf.done"
[ -d "$WINEPREFIX/dosdevices" ] || mkdir -p "$WINEPREFIX/dosdevices"

# check integrity of system directories
for system_dir in \
	   drive_c/windows/fonts \
	   drive_c/windows/inf \
	   drive_c/windows/system \
	   drive_c/windows/system32  \
  ; do 
  if [ -h "$WINEPREFIX/$system_dir" ] ; then
    if [ `stat --dereference --format %i "$WINEPREFIX/$system_dir"` = `stat --dereference --format %i "$winrar_root/Platform/$system_dir"` ] ; then
    	continue;
    fi
    [ -d "$WINEPREFIX/$system_dir" ] || rm -f "$WINEPREFIX/$system_dir"
  fi

  [ -d "$WINEPREFIX/$system_dir" ] || ln -s "$winrar_root/Platform/$system_dir" "$WINEPREFIX/$system_dir"

done

# create WinRAR Installers directory
[ -h "$WINEPREFIX/drive_c/WinRAR Installers" ] && rm -f "$WINEPREFIX/drive_c/WinRAR Installers"  
[ -d "$WINEPREFIX/drive_c/WinRAR Installers" ] || mkdir -p "$WINEPREFIX/drive_c/WinRAR Installers"  
(cd "$winrar_root/WinRAR Installers";
    for x in * ; do 
      oldlink="`pwd`/$x"
      newlink="$WINEPREFIX/drive_c/WinRAR Installers/$x"
      if [ -h "$newlink" ] ; then
	    if [ `stat --dereference --format %i "$oldlink"` = `stat --dereference --format %i "$newlink"` ] ; then
		continue;
	    fi
        [ -d "$newlink" ] || rm -f "$newlink"
      fi

      [ -d "$newlink" ] || ln -s "$oldlink" "$newlink"

    done 
)

# create wine emulated devices
if [ -h "$WINEPREFIX/dosdevices/c:" ] ; then
  [ -d "$WINEPREFIX/dosdevices/c:" ] || rm -f "$WINEPREFIX/dosdevices/c:"
fi
[ -d "$WINEPREFIX/dosdevices/c:" ] || ln -s "$WINEPREFIX/drive_c/" "$WINEPREFIX/dosdevices/c:"

if [ -h "$WINEPREFIX/dosdevices/z:" ] ; then
  [ -d "$WINEPREFIX/dosdevices/z:" ] || rm -f "$WINEPREFIX/dosdevices/z:"
fi
[ -d "$WINEPREFIX/dosdevices/z:" ] || ln -s / "$WINEPREFIX/dosdevices/z:"

if [ -h "$WINEPREFIX/dosdevices/y:" ] ; then
  [ -d "$WINEPREFIX/dosdevices/y:" ] || rm -f "$WINEPREFIX/dosdevices/y:"
fi
if [ -d "$HOME" ] ; then
    [ -d "$WINEPREFIX/dosdevices/y:" ] || ln -s "$HOME" "$WINEPREFIX/dosdevices/y:"
fi

# hide error/stdout message
if test "z$WINEDEBUG" = z ; then
    exec >/tmp/winrar.output 2>&1
fi

# install inf
for infcmd in wine.inf/DefaultInstall ; do
    inffile=`echo $infcmd | cut -d '/' -f 1`
    inftype=`echo $infcmd | cut -d '/' -f 2`

    runreg=0
    if [ -f "$WINEPREFIX/drive_c/windows/inf.done/$inffile" ] ; then
        diff -q "$WINEPREFIX/drive_c/windows/inf/$inffile" "$WINEPREFIX/drive_c/windows/inf.done/$inffile"  >/dev/null 2>&1
        if [ $? -ne 0 ] ; then
            runreg=1
        fi
    else
        runreg=1
    fi

    if [ $runreg -gt 0 ] ; then
        "${WINELOADER:-wine}" rundll32.exe setupapi.dll,InstallHinfSection $inftype 128 $inffile
        cp "$WINEPREFIX/drive_c/windows/inf/$inffile" "$WINEPREFIX/drive_c/windows/inf.done/$inffile" 
    fi
done

# run WinRAR.exe
exec "${WINELOADER:-wine}" "$winrar_root/Platform/bin/WinRARSetup.exe" "-s" "-f" "$1" "$2" "$3" "$4" "$5" "$6" "$7"

